<?php

namespace Drupal\email_contact\Controller;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\OpenModalDialogCommand;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\email_contact\Form\ContactForm;
use Drupal\Core\Entity\Entity\EntityViewDisplay;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\email_contact\Plugin\Field\FieldFormatter\EmailContactLinkFormatter;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Controller for the email_contact module.
 */
class ContactController extends ControllerBase {

  /**
   * A logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The form builder service.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * Constructs an ContactController instance.
   *
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder.
   */
  public function __construct(LoggerInterface $logger, EntityTypeManagerInterface $entity_type_manager, FormBuilderInterface $form_builder) {
    $this->logger = $logger;
    $this->entityTypeManager = $entity_type_manager;
    $this->formBuilder = $form_builder;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('logger.factory')->get('action'),
      $container->get('entity_type.manager'),
      $container->get('form_builder')
    );
  }

  /**
   * Provides a title callback for the 'email_contact.form' route.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The field host entity.
   * @param string $field_name
   *   The field name.
   * @param string $view_mode
   *   The view mode.
   *
   * @return \Drupal\Component\Render\MarkupInterface|string
   *   The route title as translated markup of string.
   */
  public function getTitle(EntityInterface $entity, string $field_name, string $view_mode) {
    /** @var \Drupal\Core\Entity\Display\EntityViewDisplayInterface $view_display */
    $view_display = $this->entityTypeManager->getStorage('entity_view_display')
      ->load("{$entity->getEntityTypeId()}.{$entity->bundle()}.$view_mode");

    if (!$view_display || !($component = $view_display->getComponent($field_name))) {
      throw new \InvalidArgumentException("Parameters passed to 'email_contact.form' route are not defining a valid display or field.");
    }

    if ($title = trim($component['settings']['title'])) {
      return $title;
    }

    return $this->t("@label - Email Contact", ['@label' => $entity->label()]);
  }

  /**
   * {@inheritdoc}
   */
  public function content(EntityInterface $entity, string $field_name = NULL, string $view_mode = 'full') {
    if (!$entity instanceof ContentEntityBase) {
      throw new NotFoundHttpException();
    }
    $view_display = EntityViewDisplay::collectRenderDisplay($entity, $view_mode);
    $field_display = $view_display->getComponent($field_name);
    $field_settings = EmailContactLinkFormatter::defaultSettings();
    if (isset($field_display['settings'])) {
      $field_settings = $field_display['settings'];
    }

    $form = $this->formBuilder->getForm(new ContactForm(
      $entity->getEntityTypeId(),
      $entity->id(),
      $field_name,
      $field_settings
    ));

    if ($field_settings['modal']) {
      return (new AjaxResponse())->addCommand(
        new OpenModalDialogCommand(
          $this->getTitle($entity, $field_name, $view_mode),
          $form,
          ['width' => '700']
        )
      );
    }

    return $form;
  }

}
